SystemOrganization addCategory: #'Seaside-Slime'!

GRSlimeBlockLintRule subclass: #WACallSuperImplementationRule
	instanceVariableNames: 'classesAndSelectors superSelectors'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WACallSuperImplementationRule methodsFor: 'running' stamp: 'lr 7/5/2010 07:44'!
checkClass: aContext
	| current |
	superSelectors := Set new.
	aContext selectedClass isMeta
		ifTrue: [ ^ self ].
	current := aContext selectedClass superclass.
	[ current isNil ] whileFalse: [
		classesAndSelectors
			at: current name
			ifPresent: [ :values | superSelectors addAll: values ].
		current := current superclass ]! !

!WACallSuperImplementationRule methodsFor: 'running' stamp: 'lr 11/8/2009 17:53'!
checkMethod: aContext
	aContext selectedClass isMeta
		ifTrue: [ ^ self ].
	((superSelectors includes: aContext selector)
		and: [ (aContext superMessages includes: aContext selector) not ])
			ifTrue: [ result addClass: aContext selectedClass selector: aContext selector ]! !

!WACallSuperImplementationRule methodsFor: 'accessing' stamp: 'lr 2/28/2009 22:07'!
group
	^ 'Seaside'! !

!WACallSuperImplementationRule methodsFor: 'initialization' stamp: 'lr 7/5/2010 07:45'!
initialize
	super initialize.
	superSelectors := Set new.
	classesAndSelectors := Dictionary new.
	classesAndSelectors
		at: #WAObject put: #( initialize postCopy );
		at: #WAPainter put: #( updateRoot: updateUrl: );
		at: #WAPresenter put: #( updateStates: initialRequest: );
		at: #WABrush put: #( setParent:canvas: with: );
		at: #WATagBrush put: #( before after openTag closeTag );
		at: #WARequestFilter put: #( handleFiltered: updateStates: );
		at: #WASession put: #( updateRoot: updateUrl: updateStates: )! !

!WACallSuperImplementationRule methodsFor: 'accessing' stamp: 'lr 2/28/2009 22:07'!
name
	^ 'Call super implementation'! !

!WACallSuperImplementationRule methodsFor: 'accessing' stamp: 'lr 2/28/2009 22:09'!
rationale
	^ 'Some template methods provided by the framework require to call super to work correctly.'! !

GRSlimeBlockLintRule subclass: #WAChangesStateWhileRenderingRule
	instanceVariableNames: 'matcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAChangesStateWhileRenderingRule methodsFor: 'running' stamp: 'lr 2/24/2009 16:09'!
checkMethod: aContext 
	| matches vars |
	aContext isRenderingMethod 
		ifFalse: [ ^ self ].
	matches := matcher 
		executeTree: aContext parseTree
		initialAnswer: OrderedCollection new.
	matches isEmpty
		ifTrue: [ ^ self ].
	vars := aContext instVarNames.
	matches do: [ :each | 
		(vars includes: each name) ifTrue: [ 
			result 
				addClass: aContext selectedClass
				selector: aContext selector.
			result matcher 
				matches: each name , ' := ``@obj'
				do: [ :node :answer | 
					answer isNil 
						ifTrue: [ node ]
						ifFalse: [ answer ] ] ] ]! !

!WAChangesStateWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside'! !

!WAChangesStateWhileRenderingRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matches: '`var'
		do: [ :node :answer | 
			(node isWrite and: [ (WASlime isWithinCallback: node) not ])
				ifTrue: [ answer add: node ].
			answer ]! !

!WAChangesStateWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Changes state while rendering'! !

!WAChangesStateWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Application state should not be changed in the rendering code, use a callback to define state.'! !

!WAChangesStateWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
resultClass
	^ ParseTreeEnvironment! !

GRSlimeBlockLintRule subclass: #WADoNotUseWhileRenderingRule
	instanceVariableNames: 'matcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WADoNotUseWhileRenderingRule methodsFor: 'running' stamp: 'lr 2/24/2009 16:15'!
checkMethod: aContext 
	| matches |
	aContext isRenderingMethod 	
		ifFalse: [ ^ self ].
	matches := matcher 
		executeTree: aContext parseTree
		initialAnswer: OrderedCollection new.
	matches do: [ :each | 
		result 
			addClass: aContext selectedClass
			selector: aContext selector.
		result addSearchString: each selector ]! !

!WADoNotUseWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside'! !

!WADoNotUseWhileRenderingRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matchesAnyOf: (self selectors
			collect: [ :each | 'self' , (self genericPatternForSelector: each) ])
		do: [ :node :answer |
			(WASlime isWithinCallback: node)
				ifFalse: [ answer add: node ].
			answer ].! !

!WADoNotUseWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Do not use while rendering'! !

!WADoNotUseWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Certain functionaly should not be used while rendering. For example: #call: and #answer: should only be used from within callback code.'! !

!WADoNotUseWhileRenderingRule methodsFor: 'private' stamp: 'jok 8/28/2011 08:08'!
selectors
	^ #( answer answer: call: show: show:onAnswer: show:onAnswer:delegation: lightbox: chooseFrom: chooseFrom:caption: confirm: inform: request: request:default: request:label: request:label:default: wait: addDecoration: addMessage: answer answer: authenticateWith:during: call: chooseFrom: chooseFrom:caption: chooseFrom:caption:onAnswer: chooseFrom:onAnswer: confirm: confirm:onAnswer: decorateWith:during: filterWith:during: home inform: inform:onAnswer: isolate: lightbox: removeDecoration: removeDelegation request: request:default: request:default:onAnswer: request:label: request:label:default: request:label:default:onAnswer: request:label:onAnswer: request:onAnswer: show: show:onAnswer: show:onAnswer:delegation: validateWith: wait: )! !

GRSlimeBlockLintRule subclass: #WADoNotUseWithinCallbackRule
	instanceVariableNames: 'matcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WADoNotUseWithinCallbackRule methodsFor: 'running' stamp: 'lr 2/24/2009 16:16'!
checkMethod: aContext
	| matches |
	aContext isRenderingMethod 	
		ifFalse: [ ^ self ].
	matches := matcher 
		executeTree: aContext parseTree
		initialAnswer: OrderedCollection new.
	matches do: [ :each | 
		result 
			addClass: aContext selectedClass
			selector: aContext selector.
		result addSearchString: each selector ]! !

!WADoNotUseWithinCallbackRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside'! !

!WADoNotUseWithinCallbackRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matchesAnyOf: (self selectors
			collect: [ :each | '``@receiver' , (self genericPatternForSelector: each) ])
		do: [ :node :answer | 
			(WASlime isWithinCallback: node)
				ifTrue: [ answer add: node ].
			answer ]! !

!WADoNotUseWithinCallbackRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Do not use within callback'! !

!WADoNotUseWithinCallbackRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Certain functionaly should not be used within callbacks. For example: #addLoadScript: should only be used while rendering.'! !

!WADoNotUseWithinCallbackRule methodsFor: 'private' stamp: 'lr 2/27/2009 21:44'!
selectors
	^ #( addLoadScript: )! !

GRSlimeBlockLintRule subclass: #WAInstantiatesComponentWhileRenderingRule
	instanceVariableNames: 'matcher'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAInstantiatesComponentWhileRenderingRule methodsFor: 'running' stamp: 'lr 2/24/2009 16:17'!
checkMethod: aContext 
	| matches |
	aContext isRenderingMethod 
		ifFalse: [ ^ self ].
	matches := matcher 
		executeTree: aContext parseTree 
		initialAnswer: OrderedCollection new.
	matches do: [ :each | 
		result 
			addClass: aContext selectedClass
			selector: aContext selector.
		result addSearchString: each receiver name ]! !

!WAInstantiatesComponentWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
group
	^ 'Seaside'! !

!WAInstantiatesComponentWhileRenderingRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matches: '`receiver `@message: ``@args'
		do: [ :node :answer |
			| class |
			((class := Smalltalk classNamed: node receiver name) notNil and: [ class isBehavior and: [ (WASlime class: class includesBehaviorNamed: #WAPresenter) and: [ (WASlime isWithinCallback: node) not ] ] ])
				ifTrue: [ answer add: node ].
			answer ]! !

!WAInstantiatesComponentWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
name
	^ 'Instantiates component while rendering'! !

!WAInstantiatesComponentWhileRenderingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:35'!
rationale
	^ 'Components should only be instanciated in initialization-code, callbacks or through lazy initialization.'! !

GRSlimeParseTreeLintRule subclass: #WAExtractCallbackCodeToMethodRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAExtractCallbackCodeToMethodRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAExtractCallbackCodeToMethodRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 22:57'!
initialize
	super initialize.
	self matcher
		matchesAnyOf: (WASlime callbackSelectors
			collect: [ :each | '`html `msg' , (self genericPatternForSelector: each) ])
		do: [ :node :answer |
			(answer isNil and: [ WASlime isBrushSelector: node receiver selector ]) 
				ifTrue: [
					node arguments
						detect: [ :each | each isBlock and: [ each body statements size > 1 ] ]
						ifNone: [ answer ] ]
				ifFalse: [ answer ] ]! !

!WAExtractCallbackCodeToMethodRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ 'Extract callback code to separate method'! !

!WAExtractCallbackCodeToMethodRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'For clarity rendering code and callback code should not be mixed, extract the contents of the callback block to a separate method.'! !

GRSlimeParseTreeLintRule subclass: #WAFixCallbackTempsMissingRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAFixCallbackTempsMissingRule class methodsFor: 'testing' stamp: 'lr 8/18/2009 22:38'!
isVisible
	"Hide the test in closure images."

	^ [ ] isKindOf: BlockContext! !

!WAFixCallbackTempsMissingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAFixCallbackTempsMissingRule methodsFor: 'initialization' stamp: 'lr 12/3/2009 10:47'!
initialize
	super initialize.
	matcher := RBParseTreeSearcher new.
	matcher 
		matches: '[ | `@temps | `@.statements ]'
		do: [ :node :answer | 
			| found |
			found := false.
			(answer isNil and: [ node parent isMessage ]) ifTrue: [
				((self selectors includes: node parent selector) 
					or: [ (WASlime brushSelectors includes: node parent selector)
					or: [ (WASlime attributeSelectors includes: node parent selector)
					or: [ (WASlime callbackSelectors includes: node parent selector) ] ] ]) ifFalse: [
						| dangerousNames |
						dangerousNames := Set new.
						node parent parents do: [ :parent | 
							parent isBlock ifTrue: [ dangerousNames addAll: parent argumentNames ].
							parent isSequence ifTrue: [ dangerousNames addAll: parent temporaryNames ] ].
							(dangerousNames anySatisfy: [ :each | node references: each ]) 
								ifTrue: [ found := true ] ] ].
			found ifTrue: [ node ] ifFalse: [ answer ] ]! !

!WAFixCallbackTempsMissingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ '#fixCallbackTemps possibly missing'! !

!WAFixCallbackTempsMissingRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'I don''t feel like explaining that here.'! !

!WAFixCallbackTempsMissingRule methodsFor: 'private' stamp: 'lr 2/27/2009 21:57'!
selectors
	^ #( allSatisfy: anySatisfy: at:ifAbsent: at:ifAbsentPut: at:ifPresent: count: critical: detect:ifNone: do: do:separatedBy: ensure: fixCallbackTemps fixTemps ifCurtailed: inject:into: noneSatisfy: on:do: reject: render: select: should: should:description: should:raise: should:raise:description: should:raise:whoseDescriptionDoesNotInclude:description: should:raise:whoseDescriptionIncludes:description: shouldnt: shouldnt:description: shouldnt:raise: shouldnt:raise:description: shouldnt:raise:whoseDescriptionDoesNotInclude:description: shouldnt:raise:whoseDescriptionIncludes:description: timesRepeat: use:during: with: )! !

GRSlimeParseTreeLintRule subclass: #WASendsRenderContentOnRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WASendsRenderContentOnRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WASendsRenderContentOnRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 21:10'!
initialize
	super initialize.
	self matcher
		matches: '`@object renderContentOn: `@html'
		do: [ :node :answer |
			(answer isNil
				and: [ (node methodNode selector ~= #visitPainter:)
				and: [ (node receiver isVariable and: [ node receiver name = 'super' ]) not ] ])
					ifTrue: [ node ]
					ifFalse: [ answer ] ]! !

!WASendsRenderContentOnRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ 'Sends #renderContentOn:'! !

!WASendsRenderContentOnRule methodsFor: 'accessing' stamp: 'lr 1/25/2010 22:07'!
rationale
	^ 'Client code should never send #renderContentOn: directly but only #render:.'! !

GRSlimeParseTreeLintRule subclass: #WASequencedAsynchronousCallsRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WASequencedAsynchronousCallsRule methodsFor: 'accessing' stamp: 'lr 1/4/2010 18:22'!
ajaxSelectors
	^ #(	"Prototype:"	evaluator insertion periodical periodicalEvaluator request updater
			"JQuery:"		ajax get getJson getScript post )! !

!WASequencedAsynchronousCallsRule methodsFor: 'running' stamp: 'lr 1/4/2010 18:17'!
checkMethod: aContext
	aContext isRenderingMethod 
		ifTrue: [ super checkMethod: aContext ]! !

!WASequencedAsynchronousCallsRule methodsFor: 'accessing' stamp: 'lr 1/4/2010 17:57'!
group
	^ 'Seaside'! !

!WASequencedAsynchronousCallsRule methodsFor: 'initialization' stamp: 'lr 1/4/2010 18:35'!
initialize
	super initialize.
	self ajaxSelectors do: [ :selector |
		self matcher
			matches: '`@receiver ' , selector , ' `@method: `@arguments' 
			do: [ :node :answer |
				answer isNil
					ifFalse: [ answer ]
					ifTrue: [ 
						| owner |
						owner := node isCascaded
							ifTrue: [ node parent parent ]
							ifFalse: [ node parent ].
						(owner notNil and: [ owner isMessage and: [ owner selector = #, ] ])
							ifTrue: [ owner ] ] ] ]! !

!WASequencedAsynchronousCallsRule methodsFor: 'accessing' stamp: 'lr 1/4/2010 17:57'!
name
	^ 'Sequenced asynchronous calls'! !

!WASequencedAsynchronousCallsRule methodsFor: 'accessing' stamp: 'lr 1/4/2010 18:38'!
rationale
	^ 'AJAX calls are asynchronous by default and they immediately return. Therefor it is a bug in most cases if an AJAX call is followed by other actions. If there are multiple AJAX calls try to combine them into one that is also more, or alternatively start the next request once the first one has completed.'! !

GRSlimeParseTreeLintRule subclass: #WAUnnecessaryBlockPassedToBrushRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAUnnecessaryBlockPassedToBrushRule methodsFor: 'running' stamp: 'lr 2/27/2009 22:13'!
checkMethod: aContext
	aContext isRenderingMethod 
		ifTrue: [ super checkMethod: aContext ]! !

!WAUnnecessaryBlockPassedToBrushRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAUnnecessaryBlockPassedToBrushRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 21:09'!
initialize
	super initialize.
	self matcher 
		matchesAnyOf: #(
			'`html `msg: [ ]'
			'`html `msg: [ `html text: ``@arg ]'
			'`html `msg: [ `html render: ``@arg ]' )
		do: [ :node :answer | 
			(answer isNil and: [ WASlime isBrushSelector: node selector ])
				ifTrue: [ node arguments first ]
				ifFalse: [ answer ] ].
	self matcher 
		matchesAnyOf: #(
			'`html `msg with: [ ]'
			'`html `msg with: [ `html text: ``@arg ]'
			'`html `msg with: [ `html render: ``@arg ]' )
		do: [ :node :answer | 
			(answer isNil and: [ WASlime isBrushSelector: node receiver selector ])
				ifTrue: [ node arguments first ]
				ifFalse: [ answer ] ]! !

!WAUnnecessaryBlockPassedToBrushRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ 'Unnecessary block passed to brush'! !

!WAUnnecessaryBlockPassedToBrushRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'Sending a block as argument to #with: is only needed when nesting brushes.'! !

GRSlimeParseTreeLintRule subclass: #WAUnnecessaryWithSentToBrushRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAUnnecessaryWithSentToBrushRule methodsFor: 'running' stamp: 'lr 2/27/2009 22:38'!
checkMethod: aContext
	aContext isRenderingMethod 
		ifTrue: [ super checkMethod: aContext ]! !

!WAUnnecessaryWithSentToBrushRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAUnnecessaryWithSentToBrushRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 21:09'!
initialize
	super initialize.
	self matcher
		matches: '`html `msg with: ``@arg'
		do: [ :node :answer |
			(answer isNil
				and: [ (node parent isCascade not)
				and: [ (WASlime isBrushSelector: node receiver selector)
				and: [ (WASlime isBrushSelector: node receiver selector , ':')
				and: [ (WASlime isBrushSelector: node methodNode selector) not ] ] ] ])
					ifTrue: [ node ]
					ifFalse: [ answer ] ]! !

!WAUnnecessaryWithSentToBrushRule methodsFor: 'accessing' stamp: 'lr 2/27/2009 22:40'!
name
	^ 'Unnecessary #with: sent to brush'! !

!WAUnnecessaryWithSentToBrushRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'Sending #with: is only required if attributes are specified too.'! !

GRSlimeParseTreeLintRule subclass: #WAUsesWrongRendererRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAUsesWrongRendererRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAUsesWrongRendererRule methodsFor: 'initialization' stamp: 'lr 2/24/2009 15:11'!
initialize
	super initialize.
	self matcher 
		matchesAnyOf: #(
			" JQuery: "
			'``@obj ajax html: [ :`r | | `@temps | `@.statements ]'
			'``@obj load html: [ :`r | | `@temps | `@.statements ]'
			'``@obj ajax script: [ :`r | | `@temps | `@.statements ]'
			'``@obj load script: [ :`r | | `@temps | `@.statements ]'
			'``@obj ajax text: [ :`r | | `@temps | `@.statements ]'
			'``@obj load text: [ :`r | | `@temps | `@.statements ]'
			" Scriptaculous: "
			'``@obj updater callback: [ :`r | | `@temps | `@.statements ]'
			'``@obj evaluator callback: [ :`r | | `@temps | `@.statements ]'
			'``@obj periodical callback: [ :`r | | `@temps | `@.statements ]'
			'``@obj autocompleter callback: [ :`r | | `@temps | `@.statements ]'
			'``@obj inPlaceEditor callback: [ :`r | | `@temps | `@.statements ]'
			'``@obj inPlaceCollectionEditor callback: [ :`r | | `@temps | `@.statements ]' )
		do: [ :node :answer | 
			(answer isNil and: [ node arguments first references: node receiver receiver name ])
				ifTrue: [ node arguments first ]
				ifFalse: [ answer ] ]! !

!WAUsesWrongRendererRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ 'Uses the wrong renderer'! !

!WAUsesWrongRendererRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'Within AJAX callbacks you need to use the renderer that is passed into the block, the renderer from the outer context is only valid during the rendering of the outer context.'! !

GRSlimeParseTreeLintRule subclass: #WAWithHasToBeLastMessageInCascadeRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAWithHasToBeLastMessageInCascadeRule methodsFor: 'running' stamp: 'lr 2/27/2009 22:13'!
checkMethod: aContext
	aContext isRenderingMethod 
		ifTrue: [ super checkMethod: aContext ]! !

!WAWithHasToBeLastMessageInCascadeRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
group
	^ 'Seaside'! !

!WAWithHasToBeLastMessageInCascadeRule methodsFor: 'initialization' stamp: 'lr 2/28/2009 21:09'!
initialize
	super initialize.
	self matcher
		matches: '`html `msg with: ``@arg'
		do: [ :node :answer | 
			(answer isNil 
				and: [ (node parent isCascade)
				and: [ (node parent messages last ~= node) 
				and: [ (WASlime isBrushSelector: node receiver selector) ] ] ]) 
					ifTrue: [ node ]
					ifFalse: [ answer ] ]! !

!WAWithHasToBeLastMessageInCascadeRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
name
	^ '#with: has to be last message in cascade'! !

!WAWithHasToBeLastMessageInCascadeRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:07'!
rationale
	^ 'Sending #with: triggers serialization of the brush attributes, any attribute being specified afterwards has no effect.'! !

GRSlimeTransformationRule subclass: #WAHandleRequestRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WAHandleRequestRule methodsFor: 'running' stamp: 'jf 3/16/2009 09:04'!
checkMethod: aContext
	(WASlime class: aContext selectedClass includesBehaviorNamed: #WARequestHandler)
		ifTrue: [ super checkMethod: aContext ]! !

!WAHandleRequestRule methodsFor: 'accessing' stamp: 'jf 12/29/2009 20:30'!
group
	^ 'Seaside 3.0'! !

!WAHandleRequestRule methodsFor: 'initialization' stamp: 'lr 11/2/2009 20:09'!
initialize
	super initialize.
	self rewriteRule
		replaceMethod: 'handleRequest: `aRequest | `@temps | `@.statements'
		withValueFrom: [ :node |
			RBParseTreeRewriter new
				replace: node arguments first name with: 'aRequestContext';
				executeTree: node.
			RBParseTreeRewriter new
				replace: 'aRequestContext' with: 'aRequestContext request';
				executeTree: node body.
			node
				selector: #handleFiltered:;
				arguments: (Array with: (RBVariableNode named: 'aRequestContext'));
				yourself ]! !

!WAHandleRequestRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:22'!
name
	^ 'Rename WARequestHandler>>#handleRequest: to #handleFiltered:'! !

!WAHandleRequestRule methodsFor: 'accessing' stamp: 'lr 1/25/2010 22:07'!
rationale
	^ 'Rename #handleRequest: to #handleFiltered when moving to Seaside 3.0.'! !

GRSlimeTransformationRule subclass: #WASessionExpiredRule
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

!WASessionExpiredRule methodsFor: 'running' stamp: 'jf 3/16/2009 08:56'!
checkMethod: aContext
	(WASlime class: aContext selectedClass includesBehaviorNamed: #WASession)
		ifTrue: [ super checkMethod: aContext ]! !

!WASessionExpiredRule methodsFor: 'accessing' stamp: 'jf 12/29/2009 20:30'!
group
	^ 'Seaside 3.0'! !

!WASessionExpiredRule methodsFor: 'initialization' stamp: 'lr 1/25/2010 22:07'!
initialize
	super initialize.
	self rewriteRule
		replaceMethod: 'expired
			| `@temps | 
			`@.Statements'
		with: 'unregistered
			| `@temps | 
			`@.Statements'! !

!WASessionExpiredRule methodsFor: 'accessing' stamp: 'lr 2/24/2009 15:22'!
name
	^ 'Rename WASession>>#expired to #unregistered'! !

!WASessionExpiredRule methodsFor: 'accessing' stamp: 'lr 1/25/2010 22:07'!
rationale
	^ 'Rename #expired to #unregisterd when moving to Seaside 3.0.'! !

!GRPackage class methodsFor: '*seaside-slime' stamp: 'lr 2/17/2010 15:22'!
seasideSlime
	^ self new
		name: 'Seaside-Slime';
		description: 'Code critis for Seaside. Detects common types of Seaside-specific bugs.';
		addDependency: 'Grease-Slime';
		url: #seasideUrl;
		yourself! !

Object subclass: #WASlime
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Seaside-Slime'!

WASlime class
	instanceVariableNames: 'brushSelectors callbackSelectors attributeSelectors'!

WASlime class
	instanceVariableNames: 'brushSelectors callbackSelectors attributeSelectors'!

!WASlime class methodsFor: 'accessing' stamp: 'lr 2/28/2009 23:09'!
attributeSelectors
	^ attributeSelectors ifNil: [ attributeSelectors := self findAttributeSelectors ]! !

!WASlime class methodsFor: 'accessing' stamp: 'lr 2/28/2009 23:09'!
brushSelectors
	^ brushSelectors ifNil: [ brushSelectors := self findBrushSelectors ]! !

!WASlime class methodsFor: 'accessing' stamp: 'lr 2/28/2009 23:09'!
callbackSelectors
	^ callbackSelectors ifNil: [ callbackSelectors := self findCallbackSelectors ]! !

!WASlime class methodsFor: 'testing' stamp: 'lr 7/25/2011 19:03'!
class: aClass includesBehaviorNamed: aSymbol
	"In order to avoid static dependencies to classes, we do this check by name."

	^ (Smalltalk includesKey: aSymbol) and: [ aClass includesBehavior: (Smalltalk at: aSymbol) ]! !

!WASlime class methodsFor: 'private' stamp: 'lr 11/2/2009 20:09'!
findAttributeSelectors
	| matcher result |
	matcher := RBParseTreeSearcher new.
	matcher
		matchesAnyOf: #(
			'self addClass: `@value'
			'self addStyle: `@value'
			'self addMedia: `@value'
			'self attributeAt: `@key put: `@value'
			'self attributeAt: `@key ifAbsentPut: `@block'
			'self attributes at: `@key put: `@value'
			'self attributes at: `@key ifAbsentPut: `@block'
			'self attributes at: `@key append: `@value'
			'self attributes at: `@key append: `@value separator: `@separator' )
		do: [ :context :node | true ].
	result := IdentitySet new.
	(self findClasses: #WABrush) , (self findClasses: #WAHtmlElement) do: [ :class | 
		class methodDictionary keysAndValuesDo: [ :selector :method | 
			(matcher executeTree: method parseTree initialAnswer: false) 
				ifTrue: [ result add: selector ] ] ].
	#(callback: enabled: labels:)
		do: [ :each | result add: each ].
	^ result! !

!WASlime class methodsFor: 'private' stamp: 'lr 11/2/2009 20:09'!
findBrushSelectors
	| matcher result |
	matcher := RBParseTreeSearcher new.
	matcher
		matchesAnyMethodOf: #(
			'`selector ^ self tag: `#tag'
			'`selector ^ self brush: `@obj'
			'`selector: `aBlock self `selector with: `aBlock'
			'`selector ^ `class on: self' )
		do: [ :context :node | true ].
	result := IdentitySet new.
	(self findClasses: #WACanvas) do: [ :class | 
		class methodDictionary keysAndValuesDo: [ :selector :method | 
			(matcher executeTree: method parseTree initialAnswer: false) 
				ifTrue: [ result add: selector ] ] ].
	^ result! !

!WASlime class methodsFor: 'private' stamp: 'lr 12/23/2009 12:52'!
findCallbackSelectors
	^ #(callback: callback:json: callback:value: callback:passengers: callbackAccordion: callbackDraggable: callbackDroppable: callbackResizable: callbackSlider: callbackSortable: callbackTabs: defaultAction: html: respond: script: text: triggerArgument:callback: triggerAutocompleter: triggerInPlaceEditor: triggerPassenger: triggerSliderCallback: triggerSortable:callback: triggerTree:callback:) asIdentitySet! !

!WASlime class methodsFor: 'private' stamp: 'lr 2/28/2009 23:02'!
findClasses: aSymbol
	| class |
	class := Smalltalk classNamed: aSymbol.
	^ class isNil 
		ifTrue: [ #() ]
		ifFalse: [ class withAllSubclasses asArray ]
	! !

!WASlime class methodsFor: 'initialization' stamp: 'jf 9/30/2009 00:19'!
initialize
	GRPlatform current addToStartUpList: self! !

!WASlime class methodsFor: 'testing' stamp: 'lr 2/28/2009 23:13'!
isBrushSelector: aString
	Symbol 
		hasInterned: aString
		ifTrue: [ :symbol | ^ self brushSelectors includes: symbol ].
	^ false! !

!WASlime class methodsFor: 'testing' stamp: 'lr 2/28/2009 23:13'!
isCallbackSelector: aString 
	Symbol 
		hasInterned: aString
		ifTrue: [ :symbol | ^ self callbackSelectors includes: symbol ].
	^ false! !

!WASlime class methodsFor: 'testing' stamp: 'lr 2/28/2009 23:13'!
isWithinCallback: aNode
	| parent |
	parent := aNode parent 
		ifNil: [ ^ false ].
	(aNode isBlock 
		and: [ parent isMessage 
		and: [ self callbackSelectors includes: parent selector ] ])
			ifTrue: [ ^ true ].
	^ self isWithinCallback: parent! !

!WASlime class methodsFor: 'initialization' stamp: 'lr 2/28/2009 23:10'!
startUp
	brushSelectors := callbackSelectors := attributeSelectors := nil! !

!SmalllintContext methodsFor: '*seaside-slime' stamp: 'lr 1/3/2010 13:10'!
isRenderingMethod
	(self selector numArgs > 0 and: [ self selector beginsWith: #render ])
		ifFalse: [ ^ false ].
	^ self compiledMethod literals anySatisfy: [ :each | each isSymbol and: [ WASlime isBrushSelector: each ] ]! !
WASlime initialize!
